from random import seed, randint, choice
from itertools import permutations
from time import time

#seed(0)

def write(filename, txt):
    '''Schreibt txt in die Datei filename'''
    fd = open(filename, mode='w')
    fd.write(txt)
    fd.close()

def random_dist(n, a=10, b=99):
    '''Gibt zufällige symmetrische Distanztabelle für n Städte zurück'''
    M = [[0 for j in range(n)] for i in range(n)]
    for i in range(0, n-1):
        for j in range(i+1, n):
            M[i][j] = M[j][i] = randint(a,b)
    return M


def round_trip_length(tour, D):
    n = len(D)
    length = 0
    T = [0] + list(tour) + [0]
    distances = [D[T[i]][T[(i+1)%n]] for i in range(0, n)]
    return sum(distances)
        
    

def tsp_brute_force(D):
    n = len(D)
    start = 0
    shortest = float('inf')
    longest = 0
    for p in permutations(range(1,n)):
        length = round_trip_length(p, D)
        #print(p, length)
        if length < shortest:
            shortest = length
        if length > longest:
            longest = length
    return shortest, longest
        

    


def benchmark(n):
    '''Führt eine TSP-Laufzeitmessung mit n Städten durch,
    zeigt die Resultate an und schreibt sie in eine 
    CSV-Datei'''

    reps = 10
    txt = ''
    tmp ='Anzahl Städte;Dauer in Sekunden\n'
    print(tmp, end='')
    txt += tmp

    for k in range(6, n+1):
        for j in range(0, reps):
            total_time = 0
            D = random_dist(k)
            t1 = time()
            tsp_brute_force(D) 
            t2 = time()
            total_time += (t2-t1)
        tmp = f'{k};{total_time:0.6f}'
        print(tmp)
        txt += tmp

    write('tsp-benchmark.csv', txt)

    
    
        



n = 5

D = random_dist(n, 10, 99)    
for row in D:
    print(row)

T = list(range(1, n))

print(tsp_brute_force(D))

benchmark(12)


